/*-
 * Copyright (C)2014 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char copyright[] = "\
@(#)Copyright (C)2014 @BABOLO http://www.babolo.ru/\n\
@(#)All rights reserved.\n\
@(#)V.M "VERS;
static const char rcsid[] = "$Id: recobe.c,v 1.13 2014/09/26 14:22:25 babolo Exp $";
#endif /* not lint */

#define BLIN_COMPAT  3
#define MIFE_COMPAT  4
#define Bpars_COMPAT 3

#include <sys/types.h>
#include <sys/stat.h>
#include <sysexits.h>
#include <strings.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <babolo/BLINflag.h>
#include <babolo/parser.h>
#include "recobe.h"

int  recobe_TestOrRevert __P((const char *p));
int  recobe_PrintAndMake __P((const char *p));
int  recobe_PrintNoRecod __P((const char *p));
int  recobe_WriteAndMake __P((const char *p));
int  recobe_G711DiffConv __P((const char *p));
int  recobe_PrintFilling __P((const char *p));
int  recobe_InterMediaTo __P((const char *p));
int  recobe_EtalonCompar __P((const char *p));
int  recobe_ReCalculator __P((const char *p));
int  recobe_ReCoordinate __P((const char *p));
void recobe_p256         __P(( const char *template
                             , const char *from
                             , const char *to
                             , u_int16_t domain
                             , const u_char array[256]
                            ));

static void
/*****************************************************************************************************/
usage(int ex) {                                                                                  /****
 *****************************************************************************************************/
    fprintf( stderr
           , "recobe @BABOLO V.M "VERS"  "DATE" http://www.babolo.ru/\n"
             "Usage: recobe [-options][ infile_or_string[ oufile]]\n"
             "   -c cnv          convert infile to oufile\n"
             "   -E -X XX        use XX as env name for opt's arg\n"
             "   -f              full control log\n"
             "   -p              print printables as symbols\n"
             "   -P              fancy print\n"
             "   -q              silent\n"
             "   -s code         control code\n"
             "   -t [A[B[name]]] print table from binary stdin\n"
             "   -v[v[v...]]     verbose\n"
             "   -z TEXT         parce TEXT as arguments continuation\n"
             "   -Z TEXT         parce TEXT as arguments continuation with env substitution\n"
             "   -= mode         internal diagnostic\n"
             "      mode: 0             blin_ctl(BLIN_CTL_DUMP)\n"
             "            1             babolo_optest(OptionsString)\n"
             "            2             babolo_optext(OptionsDescriptor)\n"
             "            3             babolo_dumpopts(OptionsDescriptor)\n"
             "            4[D]          inconsistent reverse, D - domain (1 TEXT, 2 G711)\n"
             "            5[Y[Z]]       print table(s); Y, Z code names\n"
             "            S[Y[Z]]       write table(s); Y, Z code names\n"
             "            6[A[B[name]]] print norecoding table nameA2B[]\n"
             "            7{u|A|Au|uA}  inconsistent G711u, G711A, A=>u, u=>a\n"
             "            8[D]          count, D - domain\n"
             "            9[Y[Z]]       test intermediators; Y, Z code names\n"
             "            E[Y[Z]]       compare with etalons\n"
             "   -?              this text\n"
           )
    ;
    exit(ex);
}

int
/*****************************************************************************************************
 *****************************************************************************************************/
main(int argc, char **argv) {                                                                    /****
 *****************************************************************************************************
 *****************************************************************************************************/
    const char *o, *Tflag = NULL, *tflag = NULL, *cflag = NULL;
    const char *flar = "c:flpPqs:t:v=:?";
    FILE *in = stdin, *ou = stdout;
    int c, i, code, ex = EX_OK;
    u_char *table = NULL;
    babolo_opts *bos;
    ssize_t j;
    u_char *z;

    blin_ctl(BLIN_CTL_FSET, BLIN_BIT0);
    if  (!(bos = babolo_openopts(Bpars_ESYM | Bpars_SYME, 'Z'))) {
        ifBLIN_QW0("openopts");
        ex = EX_SOFTWARE;
        goto out;
    }
    if  (!!babolo_setopts(bos, Bpars_NONU, argc, argv, flar)) {
        ifBLIN_QW0("setopts");
        ex = EX_SOFTWARE;
        goto out;
    }
    code = 0;
    for (tflag = 0;;) {
        c = babolo_getopts(bos);
        if  (!c) break;
        switch (c) {
        case 'c': cflag = babolo_getoptsarg(bos);
                ; break;
        case 'f': blin_internal_flags |= RECOBE_FULLCHECK;
                ; break;
        case 'l': blin_internal_flags |= RECOBE_PRELINERA;
                ; break;
        case 'p': blin_internal_flags |= RECOBE_PRINTABLE;
                ; break;
        case 'P': blin_internal_flags |= RECOBE_PRINTATLE;
                ; break;
        case 'q': blin_ctl(BLIN_CTL_FRST, BLIN_MASK);
                ; break;
        case 's': code = babolo_getoptsnum(bos, 0);
                ; break;
        case 't': tflag = babolo_getoptsarg(bos);
                ; break;
        case 'v': BLIN_VERBOSE(blin_internal_flags);
                ; break;
        case '=': switch((Tflag = babolo_getoptsarg(bos))[0]) {
                  case '0': blin_ctl(BLIN_CTL_DUMP);      exit(EX_OK);
                  case '1': babolo_optest(flar);          exit(EX_OK);
                  case '2': babolo_optext(bos);           exit(EX_OK);
                  case '3': babolo_dumpopts(bos);         exit(EX_OK);
                  }
                ; break;
        case '?': usage(EX_OK);
        default : ifBLIN_QX0("Illegal flag -%c", bos->c);
                ; usage(EX_USAGE);
    }   }
    if  (Tflag) {
        switch (Tflag[0]) {
        case '4': ex = recobe_TestOrRevert(++Tflag); break;
        case '5': ex = recobe_PrintAndMake(++Tflag); break;
        case 'S': ex = recobe_WriteAndMake(++Tflag); break;
        case '6': ex = recobe_PrintNoRecod(++Tflag); break;
        case '7': ex = recobe_G711DiffConv(++Tflag); break;
        case '8': ex = recobe_PrintFilling(++Tflag); break;
        case '9': ex = recobe_InterMediaTo(++Tflag); break;
        case 'E': ex = recobe_EtalonCompar(++Tflag); break;
        case 'R': ex = recobe_ReCalculator(++Tflag); break;
        case 'C': ex = recobe_ReCoordinate(++Tflag); break;
        }
    } else if (!!code) {
        j = 0;
        if  (!(z = babolo_hextrin(babolo_getargs(bos), &j))) {
            ifBLIN_QW0("babolo_hextrin");
            ex = EX_IOERR;
            goto out;
        }
        ifBLIN_QO1 blin_dumb(1, z, j);
        if  (0 > code) j = -j;
        switch ((0 > code) ? -code : code) {
        case 1: printf("csum1 = %02X\n", recobe_csum1(z, j));
                break;
        case 2: printf("crc24 = %06X\n", recobe_crc24(z, j));
                break;
        case 3: printf("crc32 = %08X\n", recobe_crc32(z, j));
                break;
        default:
                ifBLIN_QX0("Illegal code %d", code);
        }
    } else {
        if  ((o = babolo_getargs(bos)) && !(in = fopen(o, "r"))) {
            ifBLIN_QW0("fopen in %s", o);
            ex = EX_IOERR;
            goto out;
        }
        if  ((o = babolo_getargs(bos)) && !(ou = fopen(o, "w"))) {
            ifBLIN_QW0("fopen ou %s", o);
            ex = EX_IOERR;
            goto out;
        }
        if  (!!cflag) {
            if  (!(table = recobe_collection.get(cflag, 0))) {
                ifBLIN_QX0("No recoding %s", argv[1]);
                ex = EX_USAGE;
                usage(ex);
            }
            while (0 <= (i = getc(in))) {
                i &= 0xFF;
                j = table[i];
                if  (i & !j) j = '?';
                putc(j, ou);
            }
            free(table);
        } else if (!!tflag) {
            char f[2] = "0", t[2] = "0";

            f[0] = tflag[0];
            t[0] = tflag[1];
            table = malloc(256);
            for (i = 0; i < 256; ++i) {
                table[i] = getc(in);
            }
            recobe_p256(tflag + 2, f, t, RECOBE_TEXT, table);
        }
        fclose(in);
        fclose(ou);
    }
out:
    exit(ex);
}
